/*
* JBoss, Home of Professional Open Source
* Copyright 2005, JBoss Inc., and individual contributors as indicated
* by the @authors tag. See the copyright.txt in the distribution for a
* full listing of individual contributors.
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/
package org.jboss.security.federation.service;

import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;
import java.io.InputStream;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import org.apache.log4j.Logger;
import org.jboss.security.util.Tools;

import org.jboss.security.federation.model.Partner;
import org.jboss.security.sso.util.SSOUtil;


/**
 * Federation provides an implementation of the FederationService that can be managed via the jmx-console
 * 
 * @author Sohil Shah - sohil.shah@jboss.com - May 28, 2006
 *
 */
public class Federation implements FederationService
{
    private static final Logger log = Logger.getLogger(Federation.class);
    
    /**
     * xml constants
     */
    private static final String federationServer = "federation-server";
    private static final String partnersTag = "partners";
    private static final String partnerTag = "partner";
    private static final String domain = "domain";
    private static final String server = "server";
    
    private String confLocation = null;
    private Collection partners = null;
    
    /**
     * Initializes the Federation Service
     *
     */
    public Federation()
    {
        
    }
    
    /**
     * Returns a collection of partner web applications that are part of this SSO Federation
     * 
     * @return partner web applications that belong to the Single Sign On Federation
     */
    public Collection getPartners()
    {
        return this.partners;
    }
    
        
    /**
     * Location of the configuration file used to configure the Federation Service
     * 
     * @param confLocation location of the configuration file
     */
    public void setConf(String confLocation)
    {
        this.confLocation = confLocation;
    }
        
    /**
     * Starts the JMX service
     *
     */
    public void start()
    { 
        this.partners = new ArrayList();
        InputStream is = null;
        try
        {
            log.info("------------------------------");
            log.info("Conf ="+this.confLocation);
            
            
            if(!this.confLocation.startsWith("/"))
            {
                this.confLocation = "/" + this.confLocation;
            }
            
            //load the partner configuration from an xml file                            
            is = Federation.class.getResourceAsStream(this.confLocation);
                        
            //load the configuration
            Document configDocument = Tools.parseXml(is);
            
            //process the xml
            this.loadConfiguration(configDocument);
            
            log.info("------------------------------");
            log.info("Federation service successfully started...............");
        }
        catch(Exception e)
        {
            log.error(this,e);
            log.info("Federation service failed to load...............");
            this.stop();
        }
        finally
        {
            if(is!=null)
            {
                try{is.close();}catch(Exception e){}
            }            
        }
    }
    
    /**
     * Stops the JMX service
     *
     */
    public void stop()
    {       
        this.confLocation = null;
        this.partners = null;
    }
    
    /**
     * 
     * @param document
     * @throws Exception
     */
    private void loadConfiguration(Document document) throws Exception
    {
        NodeList fedServerNodes = document.getDocumentElement().getElementsByTagName(
        Federation.federationServer);
        if(fedServerNodes!=null && fedServerNodes.getLength()>0)
        {
            Element fedServer = (Element)fedServerNodes.item(0);
            NodeList partnersNodes = fedServer.getElementsByTagName(Federation.partnersTag);
            if(partnersNodes!=null && partnersNodes.getLength()>0)
            {
                Element partnersElement = (Element)partnersNodes.item(0);
                NodeList partnerNodes = partnersElement.getElementsByTagName(
                        Federation.partnerTag);
                if(partnerNodes!=null && partnerNodes.getLength()>0)
                {
                    for(int i=0;i<partnerNodes.getLength();i++)
                    {
                        Element partnerNode = (Element)partnerNodes.item(i);
                        
                        String domain = partnerNode.getAttribute(Federation.domain);
                        String server = partnerNode.getAttribute(Federation.server);
                        
                        log.info("Partner ="+domain+","+server);                        
                        
                        //setup the partners here
                        Partner partner = new Partner();
                        partner.setDomain(domain);
                        partner.setFederationServer(server);
                        this.partners.add(partner);
                    }
                }
            }
        }
    }
    
    /**
     * Returns an xml representation of partner web applications that are part of this SSO Federation
     * 
     * @return an xml representation of web applications that belong to the Single Sign On Federation
     */
    public String getPartnerInfo()
    {
    	try
    	{
	    	String xml = "";
	    	
	    	Collection partners = this.getPartners();
	    	if(partners!=null && !partners.isEmpty())
	    	{
	    		Map partnerMap = new HashMap();
	    		for(Iterator itr=partners.iterator();itr.hasNext();)
	    		{
	    			Partner cour = (Partner)itr.next();
	    			partnerMap.put(cour.getDomain(), cour.getFederationServer());
	    		}
	    		xml = SSOUtil.marshalPartners(partnerMap);
	    	}
	    	
	    	return xml;
    	}
    	catch(Exception e)
    	{
    		throw new RuntimeException(e);
    	}
    }
}
